/*-------------------------------------------------------------------------
 *
 * oauth-common.h
 *		Declarations for helper functions used for OAuth/OIDC authentication
 *
 * Portions Copyright (c) 1996-2025, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * src/include/common/oauth-common.h
 *
 *-------------------------------------------------------------------------
 */
#ifndef OAUTH_COMMON_H
#define OAUTH_COMMON_H

/* Name of SASL mechanism per IANA */
#define OAUTHBEARER_NAME "OAUTHBEARER"

#endif							/* OAUTH_COMMON_H */
